//
// LDASTools frameCPP - A library implementing the LIGO/Virgo frame
// specification
//
// Copyright (C) 2019 California Institute of Technology
//
// LDASTools frameCPP is free software; you may redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 (GPLv2) of the
// License or at your discretion, any later version.
//
// LDASTools frameCPP is distributed in the hope that it will be useful, but
// without any warranty or even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public
// License (GPLv2) for more details.
//
// Neither the names of the California Institute of Technology (Caltech),
// The Massachusetts Institute of Technology (M.I.T), The Laser
// Interferometer Gravitational-Wave Observatory (LIGO), nor the names
// of its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// You should have received a copy of the licensing terms for this
// software included in the file LICENSE located in the top-level
// directory of this package. If you did not, you can view a copy at
// http://dcc.ligo.org/M1500244/LICENSE
//

#ifndef FrameCPP_VERSION_8__IMPL__FrTableData_HH
#define FrameCPP_VERSION_8__IMPL__FrTableData_HH

#include "framecpp/Version8/FrameSpec.hh"

#include "framecpp/Version8/impl/FrObjectMetaData.hh"

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version8/GPSTime.hh"
#include "framecpp/Version8/STRING.hh"

#include "framecpp/Version8/FrVect.hh"

namespace FrameCPP
{
    namespace Version_8
    {
        namespace FrTableImpl
        {
            //===================================================================
            //
            //===================================================================
            typedef Previous::FrTable::FrTableNPS FrTableNPS;

            //===================================================================
            //
            //===================================================================
            typedef Version_4::FrTablePST<
                PTR_STRUCT,
                Common::SearchContainer< FrVect, &FrVect::GetName > >
                FrTablePS;

            class Data : public Impl::FrObjectMetaData< Data >,
                         public FrTableNPS,
                         public FrTablePS
            {
            public:
                typedef std::string name_type;
                typedef std::string comment_type;
                typedef INT_2U      nColumn_type;
                typedef INT_4U      nRow_type;
            };
        } // namespace FrTableImpl
    } // namespace Version_8
} // namespace FrameCPP

FR_OBJECT_META_DATA_DECLARE( FrTableImpl )

#endif /* FrameCPP_VERSION_8__IMPL__FrTableData_HH */
